function A=createA(mesh);
%
%  Input ==>  mesh is a structure with vert and simp as they are defined in an mcin.m file
%  Output ==> Sparse matrix A
%
%This function turns a mesh matrix into the corresponding finite elements
%approximation matrix A for linear elasticity.  Currently it will only work
%on a mesh whose edges consist only of horizontal and vertical line segments and a
%grid that is broken down into equal squares and then triangles with diagonals
%going from bottom left to top right.  If no arguments are supplied it used
%the mesh in mcin.m
%
try
    vert=mesh.vert;
    simp=mesh.simp;
catch
    mesh=getMesh;
    mesh=updateMesh(mesh);
    vert=mesh.vert;
    simp=mesh.simp;
end;

mu=8.2031;
lambda=10.4403;
values=[-2*mu-lambda -mu 0;
        -mu -2*mu-lambda 0;
        mu mu -mu];
hvalues=0.5*values;
diag=[6*mu+2*lambda 6*mu+2*lambda -2*mu];
hdiag=0.5*diag;
tdiag=0.75*diag;
qdiag=0.25*diag;
vertices=size(vert,1);
dirc=(vert(vertices,2)-floor(vert(vertices,2)))*100;
if vert(vertices,2)>=2 dirc=0; end;
vertices=double(uint16(vertices-dirc));
dirc=double(uint16(dirc));
vert(vertices+dirc,2)=floor(vert(vertices+dirc,2));
[simplices,nine]=size(simp);
A=spalloc(vertices*2,vertices*2,vertices*19);
for x=1:simplices
    tri=deciper(simp(x,4:9));
    processij(tri(1),tri(2),tri(4),tri(5),tri(9),tri(12));
    processij(tri(2),tri(3),tri(5),tri(6),tri(7),tri(10));
    processij(tri(1),tri(3),tri(4),tri(6),tri(8),tri(11));
end
for x=1:vertices
    processii(x,vert(x,2));
end;

function processij(v1,v2,t1,t2,or,ty)
   v1=v1+1;v2=v2+1;
   if (t1==1)||(t2==1) return
   elseif t1>=2 && t2>=2 && or==3
       A(v1,v2+vertices)=values(3,or);
       A(v2,v1+vertices)=values(3,or);
       A(v1+vertices,v2)=values(3,or);
       A(v2+vertices,v1)=values(3,or);  
   elseif t1>=2 && t2>=2 && ty>=2
       A(v1,v2)=hvalues(1,or);
       A(v2,v1)=hvalues(1,or);
       A(v1+vertices,v2+vertices)=hvalues(2,or);
       A(v2+vertices,v1+vertices)=hvalues(2,or);
       A(v1,v2+vertices)=hvalues(3,or);
       A(v2,v1+vertices)=hvalues(3,or);
       A(v1+vertices,v2)=hvalues(3,or);
       A(v2+vertices,v1)=hvalues(3,or);
   else    
       A(v1,v2)=values(1,or);
       A(v2,v1)=values(1,or);
       A(v1+vertices,v2+vertices)=values(2,or);
       A(v2+vertices,v1+vertices)=values(2,or);
       A(v1,v2+vertices)=values(3,or);
       A(v2,v1+vertices)=values(3,or);
       A(v1+vertices,v2)=values(3,or);
       A(v2+vertices,v1)=values(3,or);
   end;
end;

function processii(v1,t1);
    if t1==0
       A(v1,v1)=diag(1);
       A(v1+vertices,v1)=diag(3);
       A(v1,vertices+v1)=diag(3);
       A(v1+vertices,v1+vertices)=diag(2);
    end;
    if t1>=2
       c=uint8((t1-floor(t1))*10);
       if (c==4 || c==5)
           dga=tdiag;
           dgb=hdiag;
           if c==4 dgb=diag; end;
       elseif (c==1 || c==2)
           dga=qdiag;
           dgb=[0 0 0];
           if c==1 dgb=hdiag; end;
       else
           dga=hdiag;
           dgb=hdiag;
       end;
       A(v1,v1)=dga(1);
       A(v1+vertices,v1)=dgb(3);
       A(v1,vertices+v1)=dgb(3);
       A(v1+vertices,v1+vertices)=dga(2);
    end;
end;

%Given 3 vertices of a triangle and their opposite face types, 
% as [3 opp face types , 3 vertices], this function
%return info[3 vertices, 3 vertices types, 3 oppposite face orientations, 3 face types]
%where orient=1 for horizontal edge, 2 for vertical, 3 for diagonal
%where face type=0 inside, 1 dirchet, 2 neuman.
%also stores vertice type into variable vert.
function info=deciper(data)
   info=zeros(1,12);
   info(1:3)=data(4:6);
   info(10:12)=data(1:3);
   %for x=1:3
   %    if data(x)>=2
   %        for y=1:3 
   %            if y~=x info(y+3)=2; end;
   %        end;
   %    end;
   %    if data(x)==1
   %        for y=1:3 
   %            if y~=x info(y+3)=1; end;
   %        end;
   %    end;
   %end;
   for i=1:3
       info(3+i)=vert(info(i)+1,2);
   end;
   info(7)=orient(data(5),data(6));
   info(8)=orient(data(4),data(6));
   info(9)=orient(data(4),data(5));
   %for x=1:3
   %    vert(info(x)+1,2)=info(x+3);
   %end;
end;

%Given two vertex index numbers from vert this function returns
%orient=1 for horizontal, 2 for vertical, 3 for diagonal
%for relation of one point to the other.
%It assumes row i of vert contains vertex number i-1
function o=orient(vertexA,vertexB);
    o=0;
    if vert(vertexA+1,3)==vert(vertexB+1,3) o=2;
    elseif vert(vertexA+1,4)==vert(vertexB+1,4) o=1;
    else o=3;
    end;
end;        

end