function go(name,key1);
%
%   Input ==>  (name,key1) name is path to image file. key1 is optional
%                   Example: go('ring.bmp') or go('ring.bmp',0)
%   Output ==> Provides an interactive gui to play with elasticity
%
% This function allows the user to interact with elasticity.
% It loads and deforms a shape that you have drawn and saved as an image
% file.  For directions on how to create an image file, see instructions.pdf.
%
% Once began, this program provides
% two sliders and 2 text boxes.  The sliders control the x component and y
% component of the forces applied to force spot #1 and force spot #2.
% The numbers in the textboxes set the strength of the force. 
% Use key1==0 to enable faster drawing. Use key1==x to start slider force 
% equal to x.
%
% There are some sample shapes already drawn for you in the folder labeled
% images.  Enjoy!!  If you have any questions, email cdeotte@math.ucsd.edu
%

path(path,'files');
scale=3; %start max x component of force
scale2=3; %start max y component of force
fast=0;
try
    if key1==0
        fast=1;
    else
        scale=key1;
        scale2=key1;
    end;
end;
vert_old=0;simp_old=0;
incX=0;incY=0;
axs=[0 0 0 0];
lev=2;
refine=0;
slider_value=1;
slider_value2=1;
f1=[0,0];
f2=[0,0];
fg=clf;
%get(fg)
waiting=1;
done=0;
once=0;
sh = uicontrol(fg,'Style','slider','background',[.7 .7 .7],...
                'Max',2,'Min',0,'Value',slider_value,...
                'SliderStep',[0.05 0.2],...
                'Position',[70 5 200 20],...
                'CallBack',@slide);
eth = uicontrol(fg,'Style','edit','background',[.7 .7 .7],...
                'String',num2str(scale),...
                'Position',[10 5 50 20],...
                 'CallBack',@textbox);
sh = uicontrol(fg,'Style','slider',...
                'Max',2,'Min',0,'Value',slider_value,...
                'SliderStep',[0.05 0.2],...
                'Position',[25 60 20 200],...
                'CallBack',@slide2);
eth = uicontrol(fg,'Style','edit',...
                'String',num2str(scale),...
                'Position',[10 35 50 20],...
                 'CallBack',@textbox2);             
pbh = uicontrol(fg,'Style','pushbutton','String','IncX',...
                'Position',[15 270 40 20],...
                 'CallBack',@button);
pbh = uicontrol(fg,'Style','pushbutton','String','IncY',...
                'Position',[15 300 40 20],...
                 'CallBack',@button2);             
           
set(gcf,'CloseRequestFcn',@close);          
 try %if no name is supplied then use default.bmp
    name;
    mesh=img2mesh(name,0,1,1);
 catch
    mesh=img2mesh('images/default.bmp',0,1,1);
 end;
 try %check to see if we have a valid mesh
   if mesh==0
       % try searching in images folder.
       fprintf('Searching inside images folder...');
       mesh=img2mesh(strcat('images/',name),0,1,1);
       if mesh==0
           fprintf('\n');
           closereq;
           return;
       end;
   end;
 catch 
   fprintf('  Found image file.\n'); %since if has valid mesh line 77 throws error.
 end;
 mesh=updateMesh(mesh);
 A=createA(mesh);
 %B=inv(A);
 [L U]=lu(A);
 while ~done
 F=createF(mesh,f1,f2);
 %x=A\F;
 %x=B*F;
 E=L\F; 
 x=U\E; 
 if once 
     axs=axis;
     if incX
         inc=(axs(2)-axs(1))/4;
         axs(1)=axs(1)-inc;
         axs(2)=axs(2)+inc;
     end;
     if incY
         inc=(axs(4)-axs(3))/4;
         axs(3)=axs(3)-inc;
         axs(4)=axs(4)+inc;
     end;
     incX=0;
     incY=0;
 end;
 viewMesh( displace(mesh,x),0,axs,fast);
 
 while waiting
     pause(0.1);
 end;
 waiting=1;
 once=1;
 end;

             
             
   function slide(hObject,eventdata);
            slider_value=get(hObject,'Value');
            f2(1)=-1*scale*(slider_value-1);
            f1(1)=-1*scale*(-slider_value+1);
            waiting=0;
    end;
            
    function textbox(hObject,eventdata);
            scale=str2num(get(hObject,'String'));
            f2(1)=-1*scale*(slider_value-1);
            f1(1)=-1*scale*(-slider_value+1);
            waiting=0;
    end;
       function slide2(hObject,eventdata);
            slider_value2=get(hObject,'Value');
            f2(2)=-1*scale2*(slider_value2-1);
            f1(2)=-1*scale2*(-slider_value2+1);
            waiting=0;
    end;
            
    function textbox2(hObject,eventdata);
            scale2=str2num(get(hObject,'String'));
            f2(2)=-1*scale2*(slider_value2-1);
            f1(2)=-1*scale2*(-slider_value2+1);
            waiting=0;
    end;
    function button(hObject,eventdata);
            incX=1;
            waiting=0;
    end;
    function button2(hObject,eventdata);
            incY=1;
            waiting=0;
               
    end;
    function close(src,event);
        done=1;
        waiting=0;
        closereq;
    end;
    

end

